#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#define EVALUATION

class Reader
{
protected:
	FILE *f;
	int putBackCh;
public:
	Reader(FILE *F) : putBackCh(-1), f(F) { assert(f); }
	~Reader() { if (f) fclose(f); }
	void PutBack(char ch) { assert(putBackCh < 0); putBackCh = ch; }
	int GetCh() {
		if (putBackCh >= 0) { int ch = putBackCh; putBackCh = -1; return ch; }
		else { /*printf("!");*/ return fgetc(f); }}
	bool SkipWs(int& ch) { bool any = false; for ( ; ; ) {
		ch = GetCh();
//printf("<%d; %d>", any, ch);
		if (ch == EOF) return any;
		else if (! (isspace(ch) || ch == '\xa' || ch == '\xd')) { PutBack(ch); return any; }
		else any = true; } }
	bool SkipSpaces(int& ch) { bool any = false; for ( ; ; ) {
		ch = GetCh();
		if (ch == EOF) return any;
		else if (! (isspace(ch) && ch != '\xa' && ch != '\xd')) { PutBack(ch); return any; }
		else any = true; } }
	bool ReadInt(int& i, FILE *fRept, const int lineNo, const char *cesa);
	bool ReadString(char *dest, int maxChars, const char* extraStopChars); // stop at space or extraStopChars
	bool AtEof() { int ch = GetCh(); if (ch < 0) return true; else { PutBack(ch); return false; }}
	bool AtEol() { int ch = GetCh(); bool b = (ch == '\xa' || ch == '\xd');
		if (ch >= 0) PutBack(ch);
		return b; }
	bool ReadLn() {
		int ch = GetCh(); if (ch < 0) return false;
		if (ch == '\xa') return true;
		if (ch == '\xd') {
			ch = GetCh();
			if (ch >= 0 && ch != '\xa') PutBack(ch);
			return true; }
		PutBack(ch); return false; }
	void ReadLnA() { bool ok = ReadLn(); assert(ok); }
};

bool Reader::ReadString(char *dest, int maxChars, const char* extraStopChars)
{
	int nRead = 0;
	for ( ; ; )
	{
		int ch = GetCh();
		if (ch < 0 || isspace(ch) || strchr(extraStopChars, ch)) { dest[nRead] = 0; if (ch > 0) PutBack(ch); return true; }
		if (nRead == maxChars) { dest[nRead] = 0; return false; }
		dest[nRead++] = char(ch);
	}
}

bool Reader::ReadInt(int& i, FILE *fRept, const int lineNo, const char *cesa)
{
	i = 0;
	int nDigits = 0;
	bool negative = false;
	{int ch = GetCh(); if (ch == '-') negative = true; else PutBack(ch);}
	for ( ; ; )
	{
		int ch = GetCh();
		if (ch < 0) { if (nDigits <= 0) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Pricakoval bi stevilo, nasel sem EOF.\n", cesa, lineNo); return false; }
			else return true; }
		if (ch < int('0') || ch > int('9')) { PutBack(ch);
			if (nDigits <= 0) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Pricakoval bi stevilo, nasel sem znak 0x%02x.\n", cesa, lineNo, int(ch)); return false; }
			else return true; }
		// 2^31 = 2147483648
		//printf("i = %d\n", i);
		if (i > 214748364) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je preveliko (overflow).\n", cesa, lineNo); return false; }
		if (i < -214748364) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je premajhno (underflow).\n", cesa, lineNo); return false; }
		/*
		int j = 2147483647 - i, d = int(ch - '0');
		//printf("i = %d, j = %d, d = %d\n", i, j, d);
		if (j < d) { fprintf(fRept, "%d. stevilo je preveliko (overflow).\n", katero); return false; }
		*/
		int d = int(ch - '0');
		if (i == 214748364 && d > 7) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je preveliko (overflow).\n", cesa, lineNo); return false; }
		if (i == -214748364 && d > 8) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je premajhno (underflow).\n", cesa, lineNo); return false; }
		i *= 10;
		if (negative) i -= d; else i += d;
		nDigits++;
	}
}


// > 1 --> error
// <= 0 --> OK (# points = return value + 10)
//
bool Test(const char* fnIn, const char* fnOut, const char* fnRept, const char* fnOrigOut, int& nPoints)
{
	nPoints = 0;
	// Read the input file.
	FILE *fIn = fopen(fnIn, "rt"); assert(fIn);
	int cx, cy; {int i = fscanf(fIn, "%d %d", &cy, &cx); assert(i == 2);} assert(0 < cx && cx <= 200 && 0 < cy && cy <= 200);
	int *imgIn = new int[cx * cy], *imgOut = new int[cx * cy]; assert(imgIn); assert(imgOut);
	{for (int y = 0; y < cy; y++) for (int x = 0; x < cx; x++) { int i = fscanf(fIn, "%d", &(imgIn[y * cx + x])); assert(i == 1); }}
	fclose(fIn); 
	// Read the output file.
	FILE *fRept = fopen(fnRept, "wt"); assert(fRept);
	FILE *fOut = fopen(fnOut, "rb"); if (! fOut) { fprintf(fRept, "Ne morem odpreti izhodne datoteke \"%s\".\n", fnOut ? fnOut : "\0"); fclose(fRept); return false; }
	Reader reader(fOut);
	{for (int y = 0; y < cy; y++)
	{
		int dummy; 
		for (int x = 0; x < cx; x++) {
			reader.SkipSpaces(dummy);
			char buf[100]; sprintf(buf, "svetlosti piksla na x = %d, y = %d", x, y);
			int pix; if (! reader.ReadInt(pix, fRept, y + 1, buf)) { fclose(fRept); return false; }
			if (pix < 0 || pix > 255) { fprintf(fRept, "Napaka v izhodni sliki: piksel x = %d, y = %d ima barvo %d (morala bi biti od 0 do 255).\n", x, y, pix); fclose(fRept); return false; }
			imgOut[y * cx + x] = pix; }
		reader.SkipSpaces(dummy);
		if (y < cy - 1) { if (! reader.AtEol()) { fprintf(fRept, "V %d. vrstici izhodne datoteke so za %d piksli se neki dodatni znaki.\n", y + 1, cx); fclose(fRept); return false; } }
		reader.ReadLn();
	}}
	{int dummy; reader.SkipWs(dummy);} if (! reader.AtEof()) { fprintf(fRept, "Za %d vrsticami s podatki o sliki so se neki dodatni znaki.\n", cy); fclose(fRept); return false; }
	// Compare the files.
	int minDest[256], maxDest[256], hist[256]; bool hasIn[256];
	{for (int c = 0; c < 256; c++) { minDest[c] = 256; maxDest[c] = -1; hist[c] = 0; hasIn[c] = false; }}
	{for (int i = 0; i < cx * cy; i++) {
		int ci = imgIn[i], co = imgOut[i];
		hist[co]++; hasIn[ci] = true;
		if (co < minDest[ci]) minDest[ci] = co;
		if (co > maxDest[ci]) maxDest[ci] = co; }}
	int minHist = cx * cy + 1, maxHist = -1, minHistWhere = -1, maxHistWhere = -1;
	{for (int c = 0; c < 256; c++) 
	{
		if (hist[c] < minHist) minHist = hist[c], minHistWhere = c;
		if (hist[c] > maxHist) maxHist = hist[c], maxHistWhere = c;
		bool hasMin = (0 <= minDest[c] && minDest[c] < 256);
		bool hasMax = (0 <= maxDest[c] && maxDest[c] < 256);
		if (! hasIn[c]) { assert(! hasMin); assert(! hasMax); continue; }
		assert(hasMin && hasMax);
		assert(minDest[c] <= maxDest[c]);
		int cc = c + 1; while (cc < 256 && ! hasIn[cc]) cc++;
		if (cc < 256) 
		{
			assert(0 <= minDest[cc] && minDest[cc] <= maxDest[cc] && maxDest[cc] <= 256);
			if (minDest[cc] < maxDest[c]) { fprintf(fRept, "Na vhodni sliki obstaja piksel barve %d, ki je na izhodni sliki barve %d; in obenem obstaja na vhodni sliki piksel barve %d, ki je na izhodni sliki barve %d.\n", c, maxDest[c], cc, minDest[cc]); fclose(fRept); return false; }
		}
	}}
	assert(0 <= minHist && minHist <= maxHist);
	if (maxHist - minHist > 1) { fprintf(fRept, "Na izhodni sliki je %d pikslov barve %d in kar %d pikslov barve %d.\n", minHist, minHistWhere, maxHist, maxHistWhere); fclose(fRept); return false; }
	nPoints = 10; 
	fprintf(fRept, "OK, izhodna slika ustreza zahtevam naloge.\n");
	fclose(fRept);
	return true;
}

int main(int argc, char** argv)
{
	assert(argc == 5);
	int nPoints = 0;
	bool ok = Test(argv[1], argv[2], argv[3], argv[4], nPoints);
	if (! ok) return -10; else return nPoints - 10;
}
